import yaml
import inspect
import pytz
import os
import sys
import platform
import uuid
from datetime import datetime, timedelta
import json

from flask import session, request, redirect, url_for
from flask_login import LoginManager, current_user, logout_user
from sqlalchemy.orm import joinedload, sessionmaker
from sqlalchemy import desc, create_engine
import logging



from .k2obj import K2Obj
from .k2secur import K2Secur
from .k2path import K2Path
from .k2path import K2Path
from .k2datasync import K2Datasync
from flask_sqlalchemy import SQLAlchemy

class K2(K2Obj):
    name = 'k2cfg'
    version = '2.5.1.44'
    port = ''
    domain_name = ''
    domain = ''
    update_domain = ''
    components_install = []
    menu = []
    menu_url = []
    menu_category = []
    sglalchemy_pool_size = 15
    sglalchemy_max_overflow = 30
    safe_mode = False
    json_sort_keys = False
    babel_translation_directories = ''
    default_language = 'uk'
    current_language = 'uk'
    login_manager = LoginManager()
    timezone = pytz.timezone('Europe/Kiev')
    secret_key = 'JH&INH987gFDHdsagh&8dwbjdw8ckw'
    update_token = ''
    generate_migration = ''
    upgrade_migrations = ''
    platform = ''
    user_ip = ''
    user_agent = ''
    languages = {
        'en': 'English',
        'uk': 'Українська',
        'pl': 'Polski'
    }
    proj_config = yaml.safe_load(open("../proj.yml"))['proj']
    db = ''
    db_driver = ''
    class_dict = {}
    adm_menu_items = [
        {
            "title": "t_update_system",
            "icon": {"icon": "mdi mdi-cloud-download"},
            "children": [
                {
                    "title": "t_install_new_components2",
                    "to": "/components-add"
                },
                {
                    "title": "t_installed_components2",
                    "to": "/components-list"
                },
                {
                    "title": "t_update_component",
                    "to": "/components-update"

                },
                {
                    "title": "Синхронізувати метадані",
                    "to": "/components-update-metadata"

                }

            ]
        }
    ]
    adm_menu_items_category = [
        {
            "title": 't_update_system',  # назва модуля
            "to": "/components-add",  # посилання на головну сторінку модуля
            "icon": "k2",  # назва іконки модуля
            "sort": 100
        }
    ]

    # шляхи до файлу з підключенням до БД
    path_db = f"../cfg/{proj_config}/db"
    path_db_user = f"../cfg/k2/db/users/"
    db_login = None

    search_comp = [
        f"../cfg/{proj_config}/components",
        f"../usr/{proj_config}/components",
        f"components"
    ]
    app_root_path = ''
    template = 'app_stack'
    authorized_users = {}
    authorized_users_file = '../data/k2_cloud_erp/users/authorized_users.json'


    def __init__(self, *args, **kwargs):
        super().__init__()
        self.current_user_login = self.get_current_user()
        self.current_user_class = current_user
        self.secur = K2Secur()
        self.path = K2Path()
        self.data = K2Datasync()
        #self.db = None

        #self.menu = self.search_menu_items()



    def init_db_uri(self):
        '''Пошук і ініціалізація файлу з підключення до бази даних '''
        #for path in self.path_db:
        try:
            db_file_path = self.path_db + '/db.yml'
            if os.path.exists(db_file_path) and os.path.getsize(db_file_path) > 0:
                with open(db_file_path, 'r') as f:
                    db_config = yaml.load(f, Loader=yaml.FullLoader)
                return db_config
            return None
        except Exception as e:
            logging.error(f"{e}")
            return None


    def init_db_uri_user(self):
        '''Пошук і ініціалізація файлу з підключення до бази даних для поточного користуача'''
        try:
            db_file_path = self.path_db_user + self.get_current_user().get_id() + '/db.yml'
            if os.path.exists(db_file_path) and os.path.getsize(db_file_path) > 0:
                with open(db_file_path, 'r') as f:
                    db_config = yaml.load(f, Loader=yaml.FullLoader)
                return db_config
            return None
        except Exception as e:
            logging.error(f"{e}")
            return None


    # creating a connections

    def init_db(self):
        '''Підключення db_uri для вибраної бази даних'''
        db_config = self.init_db_uri()
        if db_config is None:
            db_path = 'database.db'
            db_uri = f"sqlite:///{db_path}"
        else:
            #print(db_config)
            db_uri = f"{db_config['driver']}://{db_config['user']}:{db_config['password']}@{db_config['host']}:{db_config['port']}/{db_config['dbname']}"
        return db_uri


    def init_db_user(self):
        '''Підключення db_uri для користувача'''
        from app import app
        if (hasattr(app, 'login_manager')
                and isinstance(app.login_manager, LoginManager)
                and current_user
                and K2.db_login):
            self.create_db_file_config_user()
            db_config = self.init_db_uri_user()
            db_uri = f"{db_config['driver']}://{db_config['user']}:{db_config['password']}@{db_config['host']}:{db_config['port']}/{db_config['dbname']}"
            # engine = create_engine(db_uri)
            return db_uri


    def create_db_role(self, user_name, password):
        '''Створення користувача на рівні БД'''
        try:
            import psycopg2
            db_config = K2().init_db_uri()

            # Підключення до бази даних
            conn = psycopg2.connect(
                dbname=db_config['dbname'],
                user=db_config['user'],
                password=db_config['password'],
                host=db_config['host'],
                port=db_config['port']
            )

            # Створення курсора
            cur = conn.cursor()

            # Перевірка наявності користувача
            cur.execute("SELECT 1 FROM pg_roles WHERE rolname = %s", (user_name,))
            user_exists = cur.fetchone()
            if user_exists:
                return "Користувач вже існує"
            else:
                # Виконання SQL-запиту для створення користувача
                cur.execute(f"CREATE USER \"{user_name}\" WITH PASSWORD %s", (password,))
                cur.execute(f"GRANT ALL PRIVILEGES ON DATABASE \"{db_config['dbname']}\" TO \"{user_name}\"")
                cur.execute(f"ALTER USER \"{user_name}\" WITH SUPERUSER")

            # Підтвердження змін
            conn.commit()

            # Закриття курсора та з'єднання
            cur.close()
            conn.close()

            return "Користувач успішно створений"
        except Exception as e:
            logging.error(f"Помилка створення користувача {e}")
            return "Помилка створення користувача"


    def drop_db_role(self, user_name):
        '''Видалення користувача на рівні БД'''
        try:
            import psycopg2
            db_config = K2().init_db_uri()

            # Підключення до бази даних
            conn = psycopg2.connect(
                dbname=db_config['dbname'],
                user=db_config['user'],
                password=db_config['password'],
                host=db_config['host'],
                port=db_config['port']
            )

            # Створення курсора
            cur = conn.cursor()

            # Перевірка наявності користувача
            cur.execute("SELECT 1 FROM pg_roles WHERE rolname = %s", (user_name,))
            user_exists = cur.fetchone()
            if not user_exists:
                return "Користувач не існує"
            else:
                # Виконання SQL-запиту для видалення користувача
                cur.execute(f"REVOKE ALL PRIVILEGES ON ALL TABLES IN SCHEMA public FROM \"{user_name}\"")
                cur.execute(f"DROP OWNED BY \"{user_name}\"")
                cur.execute(f"DROP USER \"{user_name}\"")

            # Підтвердження змін
            conn.commit()

            # Закриття курсора та з'єднання
            cur.close()
            conn.close()
            return "Користувач успішно видалений"
        except Exception as e:
            logging.error(f"Помилка видалення користувача {e}")
            return "Помилка видалення користувача"


    def create_db_file_config_user(self):
        '''Створення файлу з параметрами підключенням до БД для користувача'''
        db_file_path = self.path_db_user + self.get_current_user().get_id() + '/db.yml'
        db_config = K2().init_db_uri()

        from app import app
        import hashlib
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            from components.k2site.k2site.models import K2users
            user_id = current_user.get_id()
            user = K2users.query.get(user_id)
            if user is not None:
                login = user.login
                password = user.password
        password = hashlib.md5(password.encode()).hexdigest()
        # Перевірка наявності шляху та створення відсутніх каталогів
        db_dir = os.path.dirname(db_file_path)
        if not os.path.exists(db_dir):
            os.makedirs(db_dir)
        # Перевірка наявності файлу та його створення, якщо він не існує
        if not os.path.exists(db_file_path):
            with open(db_file_path, 'w') as f:
                yaml.safe_dump({}, f)  # Створення пустого файлу

        # Зчитування поточного вмісту файлу
        with open(db_file_path, 'r') as f:
            existing_data = yaml.safe_load(f) or {}

        # Оновлення поточного вмісту файлу з даними конфігурації
        existing_data.update({
            'dbname': db_config['dbname'],
            'driver': db_config['driver'],
            'user': login,
            'password': password,
            'host': db_config['host'],
            'port': db_config['port']
        })
        # # Створення словника з даними конфігурації
        # data = {
        #     'dbname': db_config['dbname'],
        #     'user': login,
        #     'password': 'k2adm123456',
        #     'host': db_config['host'],
        #     'port': db_config['port']
        # }

        # Запис оновлених даних у файл YAML
        with open(db_file_path, 'w') as f:
            yaml.safe_dump(existing_data, f)


    def db_user_engine(self, db):
        '''Зміна параметрів підключення для поточного користувача'''

        if self.current_user_class.get_login() not in db.engines:
            db.engine.dispose()
            db.engines[self.current_user_class.get_login()] = create_engine(self.init_db_user())
            #db.engines[None] = create_engine(self.init_db_user())  #
            K2Obj.db = db.engines[self.current_user_class.get_login()]
        #return db.engines[self.current_user_class.get_login()]


    def init_lm(self, app):
        '''Ініціалізація логін-менеджера'''
        # Initialize LoginManager
        self.login_manager.init_app(app)


    def get_current_user(self):
        '''Select current_user
        K2().get_current_user()'''

        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager, LoginManager):
            return current_user
        else:
            return None


    def add_authorized_users(self, user_id, login):
        '''Додати користувача до списку авторизованих'''

        # Перевірити наявність каталогу
        directory = os.path.dirname(self.authorized_users_file)
        if not os.path.exists(directory):
            os.makedirs(directory)

        # Зчитати існуючі дані з файлу, якщо він існує
        if os.path.exists(self.authorized_users_file):
            with open(self.authorized_users_file, 'r') as f:
                data = json.load(f)
        else:
            data = []

        # Перевірити, чи вже є користувач з таким user_id
        user_exists = False
        for user in data:
            if user['user_id'] == user_id:
                user_exists = True
                break

        # Якщо користувача з таким user_id ще немає, додати його
        if not user_exists:
            data.append({'user_id': user_id, 'login': login})

            # Записати оновлені дані у файл
            with open(self.authorized_users_file, 'w') as f:
                json.dump(data, f)


    def dell_authorized_users(self, user_id):
        '''Видалити користувача зі списку авторизованих'''
        if os.path.exists(self.authorized_users_file):
            with open(self.authorized_users_file, 'r') as f:
                data = json.load(f)
            # Видалити користувача зі списку
            data = [user for user in data if user['user_id'] != user_id]
            # Записати оновлені дані у файл
            with open(self.authorized_users_file, 'w') as f:
                json.dump(data, f)

    # def is_logout_authorized_users(self, user_id):
    #     '''Додавання мітки is_logout до списку авторизованих'''
    #     with open(self.authorized_users_file, 'r') as f:
    #             authorized_users = json.load(f)
    #     for user in authorized_users:
    #         if user['user_id'] == user_id:
    #             break
    #
    #     with open(self.authorized_users_file, 'w') as f:
    #         json.dump(authorized_users, f)

    def get_authorized_users(self):
        '''Отримати список авторизованих користувачів з файлу'''
        if os.path.exists(self.authorized_users_file):
            with open(self.authorized_users_file, 'r') as f:
                data = json.load(f)
            return data
        else:
            return []

        with open(self.authorized_users_file, 'w') as f:
            json.dump(authorized_users, f)


    def check_logout_users(self, user_id):
        try:
            from app import app
            if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                            LoginManager) and current_user.is_authenticated:
                from components.k2site.k2site.models import K2users
                user_id = current_user.get_id()

                user_online_list_f = self.get_authorized_users()
                user_online_list = [user['user_id'] for user in user_online_list_f]
                user = K2users.query.filter_by(user_id=user_id).first()
                if user.locale is not None:
                    K2.current_language = user.locale
                else:
                    K2.current_language = K2.default_language
                if user_id not in user_online_list and user.is_logout_by_admin == 1:
                    logout_user()
        except Exception as e:
            logging.info(e)


    def check_authorized_users(self, user_id):
        '''Перевірити чи користувач активний протягом останніх 5 хв'''
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            try:
                from components.k2site.k2site.models import K2log_auth
                last_activity  =  K2log_auth.query.filter_by(user_id=user_id).order_by(desc(K2log_auth.date_auth)).first()
                if last_activity:
                    last_activity_date = last_activity.date_auth
                    if last_activity_date:
                        # Calculate the time difference between the last activity date and the current time
                        time_difference = datetime.now() - last_activity_date
                       # Check if the time difference is greater than 5 minutes
                        if time_difference > timedelta(minutes=5):
                            self.dell_authorized_users(user_id)
            except Exception as e:
                logging.error(f'Error check_authorized_users {e}')


    # def get_macadress_for_ip(self, ip):
    #     'Returns a list of MACs for interfaces that have given IP, returns None if not found'
    #     import netifaces as nif
    #     for i in nif.interfaces():
    #         addrs = nif.ifaddresses(i)
    #         try:
    #             if_mac = addrs[nif.AF_LINK][0]['addr']
    #             if_ip = addrs[nif.AF_INET][0]['addr']
    #         except Exception as e:
    #             if_mac = if_ip = None
    #         if if_ip == ip:
    #             return if_mac
    #     return None

    # load database settings from yml file

    def get_user_project_id(self):
        '''Пошук id поточного проекту користувача
            K2().get_user_project_id()

        '''
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            from components.k2site.k2site.models import K2users
            user_id = current_user.get_id()
            user = K2users.query.get(user_id)
            if user is not None:
                current_proj_id = user.projid
            else:
                current_proj_id = None
        else:
            current_proj_id = None
        return current_proj_id


    def get_user_project_name(self):
        '''Пошук назви поточного проекту користувача
           K2().get_user_project_name()
        '''
        try:
            if self.get_user_project_id() :
                from components.k2site.k2site.models import K2Proj
                projid = self.get_user_project_id()
                project = K2Proj.query.get(projid)
                if project:
                    projname = project.projname
                else:
                    projname = None
            else:
                projname = None
            return projname
        except Exception as e:
            logging.error(f"{e}")
            return None


    def get_user_counterparts_id(self):
        '''Пошук id поточного контрагента
           K2().get_user_counterparts_id()
           '''
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            from components.k2site.k2site.models import K2users
            user_id = current_user.get_id()
            user = K2users.query.get(user_id)
            if user is not None:
                current_counterpart_id = user.counterpart_id
            else:
                current_counterpart_id = None
        else:
            current_counterpart_id = None
        return current_counterpart_id


    def get_user_counterparts_name(self):
        '''Пошук назви поточного контрагента
           K2().get_user_counterparts_name()
        '''
        try:
            if self.get_user_counterparts_id() :
                from components.k2handbook.k2handbook.models import k2counterparts
                counterparts_id = self.get_user_counterparts_id()
                counterparts = k2counterparts.query.get(counterparts_id)
                if counterparts:
                    counterparts_name = counterparts.counterpart_name
                else:
                    counterparts_name = '-'
            else:
                counterparts_name = '-'
            return counterparts_name
        except Exception as e:
            logging.error(f"{e}")
            counterparts_name = '-'
            return counterparts_name


    def get_user_storage_id(self):
        '''Пошук id поточного складу
            K2().get_user_storage_id()
        '''
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            from components.k2site.k2site.models import K2users
            user_id = current_user.get_id()
            user = K2users.query.get(user_id)
            if user is not None:
                current_storage_id = user.storage_id
            else:
                current_storage_id = None
        else:
            current_storage_id = None
        return current_storage_id


    def get_user_stoages_name(self):
        '''Пошук назви поточного складу
            K2().get_user_stoages_name()
        '''
        try:
            if self.get_user_storage_id() :
                from components.k2handbook.k2handbook.models import k2storage
                storage_id = self.get_user_storage_id()
                storages = k2storage.query.get(storage_id)
                if storages:
                    storages_name = storages.storage_name
                else:
                    storages_name = '-'
            else:
                storages_name = '-'
            return storages_name
        except Exception as e:
            logging.error(f"{e}")
            storages_name = '-'
            return storages_name


    def get_project_setting(self):
        '''Форма зміни проекту'''
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            try:
                from components.k2site.k2site.views import K2Site
                grid_settings = K2Site.show_projects()
                return grid_settings
            except Exception as e:
                logging.error(f"{e}")
                grid_settings = ''
                return grid_settings

    def get_all_subclasses(cls):
        subclasses = K2Obj
        for subclass in list(subclasses):
            subclasses.extend(cls.get_all_subclasses(subclass))
        return subclasses

    def search_class_dict(self, parent_class=None):
        '''Рекурсивний пошук властивостей нащадків базового класу'''

        if parent_class is None:
            parent_class = K2Obj

        for subclass in parent_class.__subclasses__():
            class_name = subclass.__name__.lower()
            K2.class_dict[class_name] = {}
            # Додаємо властивості поточного підкласу до словника
            if hasattr(subclass, 'adm_menu_items'):
                K2.class_dict[class_name]['adm_menu_items'] = subclass.adm_menu_items
            if hasattr(subclass, 'adm_menu_items_category'):
                K2.class_dict[class_name]['adm_menu_items_category'] = subclass.adm_menu_items_category
            if hasattr(subclass, '_path_class'):
                K2.class_dict[class_name]['path'] = subclass._path_class
            if hasattr(subclass, 'export_data'):
                K2.class_dict[class_name]['export_data'] = subclass.export_data
            if hasattr(subclass, '_name_file_class'):
                K2.class_dict[class_name]['name_file'] = subclass._name_file_class

            # Рекурсивно шукаємо властивості нащадків поточного підкласу
            self.search_class_dict(subclass)


    def url_map(self):
        from flask import current_app
        all_routes = [
            rule.rule
            for rule in current_app.url_map.iter_rules()
        ]

        return all_routes

    def get_menu_url(self):
        '''Повертає список url актиивних пунктів меню'''
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            from .k2upd import K2Menu
            response = K2Menu().get_admin_menu()
            K2.menu_url = K2Menu().get_admin_menu_url(response)


    def search_menu_items(self):
        '''Search menu items'''
        try:
            from app import app
            if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                            LoginManager) and current_user.is_authenticated:
                from .k2upd import K2Menu
                response = K2Menu().get_admin_menu()
                K2.menu = response
                #K2.menu_url = K2Menu().get_admin_menu_url(response)
                #url_map all routes
                # K2.menu_url = [
                #     rule
                #     for rule in current_app.url_map.iter_rules()
                # ]
            else:
                K2.menu = []
                K2.menu_url = []
        except Exception as e:
            logging.error(f"search_menu_items {e}")


    def search_menu_items_category(self):
        '''Search menu items category'''
        try:
            from app import app
            if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                            LoginManager) and current_user.is_authenticated:
                from k2.k2upd import K2Menu
                response = K2Menu().get_admin_menu_category()  # requests.get(url=f"{K2.domain_loc}/api/main-menu-category/", headers=request.headers)
                # if response.status_code == 200:
                K2.menu_category = response
                # else:
                #    self.menu_category = []
            else:
                K2.menu_category = []
        except Exception as e:
            logging.error(f"{e}")


    def current_template(self):
        '''Search current template'''
        current_template_bd = None
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            from components.k2site.k2site.models import K2users
            user_id = current_user.get_id()
            user = K2users.query.get(user_id)
            if user is not None:
                current_template_bd = user.user_theme_pyt
        if current_template_bd is not None:
            return current_template_bd
        else:
            return self.template

    # def get_user_agent(self, current_request):
    #     pass


    @classmethod
    def namemenu(cls, url):
        namemenu_itm = 'k2itm' + url.replace('/', '-')
        return namemenu_itm

    @classmethod
    def get_user_role(cls, user_id):
        '''Отрмання roleid користувача '''
        try:
            from app import app
            if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                            LoginManager) and current_user.is_authenticated:
                from components.k2site.k2site.models import K2users
                from .k2admmenu import K2admin_Menus_Prava, K2admin_menus

            # Отримати roleid
            user_role = K2users.query.filter_by(user_id=user_id).first().roleid
            return user_role
        except Exception as ex:
             print(ex)
             return {'error': 'Помилка при виконанні запиту до бази даних'}


    @classmethod
    def get_current_user_role_name(cls):
        '''Отрмання rolename поточного користувача '''
        try:
            from app import app
            if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                            LoginManager) and current_user.is_authenticated:
                from components.k2site.k2site.models import K2roles
            user_id = current_user.get_id()
            roleid = cls.get_user_role(user_id)
            # Отримати rolename
            user_role = K2roles.query.filter_by(roleid=roleid).first().rolename
            return user_role
        except Exception as ex:
             print(ex)
             return {'error': 'Помилка при виконанні запиту до бази даних'}



    @classmethod
    def get_user_permissions(cls, user_id, namemenu):
        '''Отрмання прав користувача залежно від url'''
        try:
            from app import app
            if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                            LoginManager) and current_user.is_authenticated:
                from components.k2site.k2site.models import K2users
                from .k2admmenu import K2admin_Menus_Prava, K2admin_menus

            # Отримати roleid
            user_role = cls.get_user_role(user_id)

            # Отримати права за вказаним roleid та namemenu
            user_permissions = K2admin_Menus_Prava.query \
                .join(K2admin_menus, K2admin_Menus_Prava.menuid == K2admin_menus.menuid) \
                .filter(K2admin_Menus_Prava.roleid == user_role, K2admin_menus.namemenu == namemenu) \
                .options(joinedload(K2admin_Menus_Prava.menu)) \
                .all()

            # Перетворити результат в словник
            if user_permissions:
                permissions_list = []
                for permission in user_permissions:
                    permissions_dict = {
                        'roleid': permission.roleid,
                        'r': permission.r,
                        'w': permission.w,
                        'i': permission.i,
                        'd': permission.d,
                        'c': permission.c,
                        'exp': permission.exp,
                        'imp': permission.imp,
                        'del_': permission.del_,
                        'settable': permission.settable,
                        'cutpast': permission.cutpast,
                        'enable': permission.enable,
                        'active': permission.active
                        # Додайте інші поля за необхідності
                    }
                    permissions_list.append(permissions_dict)
            else:
                permissions_list = None

            return permissions_list[0]
        except Exception as ex:
             print(ex)
             return {'error': 'Помилка при виконанні запиту до бази даних'}

    @classmethod
    def search_yml(cls, name_yml):
        """Search for a yml file in the caller directories of subclasses.

        This method searches for a yml file with a specified name in the caller directories of all subclasses of K2Obj.
        The search is performed in the 'yml' subdirectory of each caller directory.

        Args:
            name_yml: The name of the yml file to search for (without the '.yml' extension).

        Returns:
            The caller directory where the yml file was found, or None if the file was not found.

        Usage:
            To search for a yml file named 'users.yml', call this method as follows:
                K2.search_yml('users')
        """
        for subclass in K2Obj.__subclasses__():
            caller_path = inspect.getfile(subclass)
            caller_dir = os.path.dirname(caller_path)

            # Search for the yml file in the 'yml' subdirectory
            yml_dir = os.path.join(caller_dir, 'yml')
            yml_file = os.path.join(yml_dir, f'{name_yml}.yml')
            if os.path.exists(yml_file):
                # Return the caller directory if the yml file is found
                return caller_dir

        # Return None if the yml file is not found
        return None

    @classmethod
    def get_path_to_root(cls, caller_file):
        '''
        Відносний шлях до рута.
        Виклик  K2.get_path_to_root(__file__).
        __file__ - стандартна змінна в Python, яка містить абсолютний шлях до поточного файлу, в якому вона використовується.
        '''
        curr_dir = os.path.dirname(__file__)  # Каталог поточного файлу
        root_dir = os.path.dirname(curr_dir)  # Каталог батьківського каталогу
        relative_path = os.path.relpath(caller_file, root_dir)
        return relative_path

    @classmethod
    def get_path_abs(cls, caller_file):
        '''
        Абсолютний шлях до файлу, в якому виконується код.
        Виклик K2.get_path_abs(__file__)
        __file__ -стандартна змінна в Python, яка містить абсолютний шлях до поточного файлу, в якому вона використовується. '''
        return os.path.abspath(caller_file)

    @classmethod
    def generate_id(cls):
        '''Генерація ID'''
        generated_id = uuid.uuid4().hex[:32]
        return ''.join([generated_id[i:i + 4] for i in range(0, len(generated_id), 4)])


    @classmethod
    def load_babel_translation_directories(cls):
        '''Завантаження перекладів'''
        with open('languages/babel_translation_directories.yml', 'r') as f:
            data = yaml.safe_load(f)
            # print(data['babel_translation_directories'])
        cls.babel_translation_directories = data['babel_translation_directories']


    @classmethod
    def search_babel_translation_directories(cls):
        '''Пошук файлів з перекладами'''
        # response = requests.get(f'{cls.domain}api/languages')
        from k2.k2trans import K2Lang
        response = K2Lang.find_language()
        if response:
            cls.babel_translation_directories = response
            data = {'babel_translation_directories': response}
        else:
            cls.babel_translation_directories = 'languages'
            data = {'babel_translation_directories': 'languages'}

        with open('languages/babel_translation_directories.yml', 'w') as f:
            yaml.dump(data, f)



    @classmethod
    def get_locale(cls):
        '''Визначення поточної мови'''
        if 'lang' in session:
            # Отримати обрану мову з сеансу користувача
            return session['lang']
        elif cls.current_language != '':
            return cls.current_language
        else:
            # Повернути мову за замовчуванням з налаштувань додатку
            return cls.default_language  # app.config['DEFAULT_LANGUAGE']

    @classmethod
    def get_locale_id(cls):
        '''Визначення id поточної мови'''
        lang = cls.get_locale()
        try:
            Session = sessionmaker(bind=K2Obj.db)
            session = Session()
            from components.k2handbook.k2handbook.models import k2language

            # language = k2language.query.filter_by(language_symbol=lang).first()
            language = session.query(k2language).filter_by(language_symbol=lang).first()
            if language:
                lang_id = language.language_id
            else:
                lang_id = None
        except Exception as e:
            logging.error(f"Помилка при виконанні запиту до бази даних. {e}")
            lang_id = None
        finally:
            session.close()
        return lang_id


    @classmethod
    def get_active_lang_list(cls):
        '''Визначення id поточної мови'''
        try:
            Session = sessionmaker(bind=K2Obj.db)
            session = Session()
            from components.k2handbook.k2handbook.models import k2language
            lang_list = []
            # language = k2language.query.filter_by(language_symbol=lang).first()
            language = session.query(k2language).filter_by(language_active=1).all()
            if language:
                for lang in language:
                    lang_list.append(lang.language_symbol)
        except Exception as e:
            logging.error(f"Помилка при виконанні запиту до бази даних. {e}")
            lang_id = None
        finally:
           session.close()

        return lang_list


    @classmethod
    def get_platform(cls):
        '''Визначення операційної системи'''
        operating_system = platform.system()
        cls.platform = operating_system
        return cls.platform

    path_objs = [
        f"../cfg/{proj_config}/k2/{name.lower()}.py",
        f"cfg/{name.lower()}.py",
        f"usr/cfg/{name.lower()}.py"
    ]

    @classmethod
    def ins_search_comp(self):
        '''Вивід списку компонент з файлу components.yml у всіх шляхах пошуку '''
        result_dict = {'components': []}
        for components_list_yml in self.search_comp:
            if os.path.exists(components_list_yml):
                with open(f"{components_list_yml}/components.yml", 'r') as file:
                    components_data_itm = yaml.safe_load(file)

                    for component in components_data_itm['components']:
                        name = component['name']
                        existing_component = next(
                            (c for c in result_dict['components'] if c['name'] == name),
                            None
                        )
                        if existing_component is None:
                            result_dict['components'].append(component)
        return result_dict


    @classmethod
    def compare_versions(cls,version1, version2):
        # Розділяємо версії на числа
        parts1 = list(map(int, version1.split('.')))
        parts2 = list(map(int, version2.split('.')))

        # Порівнюємо кожну частину версії
        for i in range(min(len(parts1), len(parts2))):
            if parts1[i] < parts2[i]:
                return -1  # Версія 1 менше версії 2
            elif parts1[i] > parts2[i]:
                return 1  # Версія 1 більше версії 2

        # Якщо всі частини досягли рівняння, то версії рівні
        if len(parts1) < len(parts2):
            return -1  # Версія 1 коротше версії 2
        elif len(parts1) > len(parts2):
            return 1  # Версія 1 довше версії 2

        return 0  # Версії ідентичні

    # @staticmethod
    def component_list():
        '''Пошук встановлених компонент'''
        with open(f"components/components.yml", 'r') as file:
            components_data = yaml.safe_load(file)
        if components_data is None or 'components' not in components_data:
            components_d = []
        else:
            components_d = components_data['components']
        installed_components = [{component['id']: component['name']} for component in components_d if
                                component.get('installed', False)]
        return installed_components

    components_install = component_list()  # список встановлених компонент

    # Отримуємо значення з path_objs для розширення поточного класу
    # @staticmethod
    def search_class_prop(path_objs):
        ''''''
        code_str = None
        for path_obj in path_objs:
            if os.path.exists(path_obj):
                with open(path_obj, 'r') as file:
                    code_str = file.read()
                break  # Виходимо з циклу, якщо знайдено файл
        else:
            sys.exit()  # Завершуємо процес, якщо жоден файл не знайдено
        return code_str

    code_str = K2Obj.search_class_prop(path_objs)


    try:
        exec(code_str)
    except Exception as e:
        print(f"Error in K2cfg {e}")
        sys.exit()
    domain_loc = f"http://127.0.0.1:{port}"

# print(K2.__dict__)
# print(K2.path_objs)

