import os
from flask_login import LoginManager, current_user

from .k2obj import K2Obj


class K2Path(K2Obj):
    '''клас формування і пошуку ієрархії файлової системи проектів'''
    name = 'k2path'
    def __init__(self, *args, **kwargs):
        super().__init__()

    #     self._data_path = None
    #
    # @property
    # def data_path(self):
    #     """Getter"""
    #     return self._data_path
    #
    # @data_path.setter
    # def name_yml(self, name: str):
    #     """Setter method"""
    #     self._data_path = name

    def __current_proj(self, current_project_id):
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            from components.k2site.k2site.models import K2Proj
            current_project = K2Proj.query.filter_by(projid=current_project_id).first()
        return current_project


    def __current_usr(self):
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            from components.k2site.k2site.models import K2Proj, K2users
            user_id = current_user.get_id()
            user = K2users.query.get(user_id)
        return user


    def __current_countreparts(self):
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            from components.k2site.k2site.models import K2Proj, K2users
            user_id = current_user.get_id()
            user = K2users.query.get(user_id)
            if user is not None:
                current_counterpart_id = user.counterpart_id
            else:
                current_counterpart_id = None
        return current_counterpart_id


    def __current_stor(self):
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            from components.k2site.k2site.models import K2Proj, K2users
            user_id = current_user.get_id()
            user = K2users.query.get(user_id)
            if user is not None:
                current_storage_id = user.storage_id
            else:
                current_storage_id = None
        return current_storage_id


    def __traverse_hierarchy(self, project, project_names):
        '''Рекурсивна функція для обходу ієрархії проектів'''
        #nonlocal project_names
        parent_project = self.__current_proj(project.parentid)
        if parent_project:
            project_names.append(parent_project.projid)
            # Якщо є батьківський проект, рекурсивно викликайте функцію для нього
            self.__traverse_hierarchy(parent_project, project_names)


    def __find_path(self, current_project=None):
        '''Recursive function to determine the path to the folder
           of the user in the project hierarchy.

           Usage: K2().find_path()

           Returns: Folder hierarchy for projects for current user
        '''
        user = self.__current_usr()
        if user:
            if not current_project:
                current_project_id = user.projid  #отримали поточний проект
                current_project = self.__current_proj(current_project_id)
                project_names = [current_project.projid] if current_project else []  # Початковий список з іменем(id) поточного проекту

        # Викликати рекурсивну функцію
        self.__traverse_hierarchy(current_project, project_names)

        # Сформувати шлях, об'єднавши імена проектів від кінця до початку
        user_project_path = '/'.join(reversed(project_names))
        return user_project_path


    def data_path(self, work_path=None, user_folder=False):
        '''
        Create folder hierarchy and return path

        Args:
        work_path - str (end work_path)
        user_folder - boolean (False: defaulL , True: if need create user folder)

        Returns: str user data folder path hierarchy

        Usage: K2().path.data_path('conf') or K2().path.data_path('conf', True)

        '''
        root_path = 'data'
        user_project_path = self.__find_path()
        folder_list = ['..', root_path, user_project_path]
        user_countrepart = self.__current_countreparts()
        if user_countrepart:
            folder_list.append(user_countrepart)
        user_storages = self.__current_stor()
        if user_storages:
            folder_list.append(user_storages)
        if user_folder:
            user = self.__current_usr()
            # Отримати шлях до папки користувача
            folder_list.append('users')
            user_path = user.user_id
            folder_list.append(user_path)
        if work_path:
            folder_list.append(work_path)
        user_folder_path = os.path.join(*folder_list)
        # Перевірити існування структури папок
        if not os.path.exists(user_folder_path):
            # Якщо не існує, створити структуру папок
            os.makedirs(user_folder_path)
        return user_folder_path


    def media_path(self, work_path=None):
        '''
        Create folder hierarchy and return path for media

        Args:
        work_path - end work_path

        Returns: str user media folder path hierarchy

         Usage: K2().path.media_path()

        '''
        pass

    def __find_file_in_path(self, directory, file_name):
        '''Рекурсивна функція для пошуку файлу в директорії'''

        # Розділити абсолютний шлях на всі його складові
        path_components = directory.split(os.path.sep)

        # Поступово зменшуємо шлях, включаючи більше верхніх папок кожен раз
        for i in range(len(path_components), 0, -1):
            current_path = os.path.join(*path_components[:i])
            file_path = os.path.join(current_path, file_name)

            # Перевірка, чи файл із зазначеним ім'ям знаходиться в поточній директорії
            if os.path.isfile(file_path):
                return file_path

        # Якщо файл не знайдено в жодній з піддиректорій, повертається значення None
        return None


    def find_data_file(self, file_name):
        '''
        Search for a file in the project hierarchy structure

        Args:
        file_name - str (name of the file to be found)

        Returns: str file path

        Usage: K2().path.find_data_file('data.txt') or K2().path.find_data_file('reports/report_exp.mrt')
        '''


        search_path = []
        if  self.__current_stor():
            search_path.append('../data/' + self.__find_path() + '/' + self.__current_countreparts() + '/' + self.__current_stor() + '/users/' + self.__current_usr().user_id)

        if self.__current_countreparts():
            search_path.append('../data/' + self.__find_path() + '/' + self.__current_countreparts() + '/users/' + self.__current_usr().user_id)

        search_path.append('../data/' + self.__find_path() + '/users/' + self.__current_usr().user_id)
        search_path.append('../data/' + self.__find_path())
        for path in search_path:
            file_path = self.__find_file_in_path(path, file_name)
            if file_path:
                return file_path

        # Якщо файл не знайдено в жодному з шляхів, повертаємо None
        return None

