import requests
import shutil
import zipfile
import os
import json

from k2.k2cfg import K2

# Ваш JWT токен
token_file = '../builder/config/token.txt'
with open(token_file, 'r') as file:
    jwt_token = file.read().strip()  # Зчитуємо токен з файлу і видаляємо зайві пробіли, якщо вони є
# URL СЕРВЕРА ОНОВЛЕНЬ для завантаження файлу
server_url = K2.update_domain + '/k2update/api/save-archive-components-with-tokens/'
#server_url = 'http://127.0.0.1:7654' + '/k2update/api/save-archive-components-with-tokens/'

# конфіги
components_list_file = '../builder/config/components-list.txt'
components_list = []  # Ініціалізуємо пустий список components_list
with open(components_list_file, 'r') as file:
    for line in file:
        component_path = line.strip()  # Видаляємо зайві пробіли та символи нового рядка
        components_list.append(component_path)  # Додаємо компонент до списку components_list

#цикл по списку компонент
for comp in components_list:
    try:
        if os.path.exists(comp):
            # Визначаємо ім'я архіву, що співпадає з останньою папкою
            comp_name = os.path.basename(comp)
            zip_filename = f'../builder/components/{comp_name}.zip'
            # Створюємо папку `comp_name` та копіюємо всю папку `comp` до неї
            comp_name_folder = os.path.join('../builder/components/', comp_name)
            comp_name_folder2 = os.path.join(comp_name_folder, comp_name)
            shutil.copytree(comp, comp_name_folder2)

            # Відкриваємо файл архіву для запису
            with zipfile.ZipFile(zip_filename, 'w', zipfile.ZIP_DEFLATED) as zipf:
                # Рекурсивно додаємо всю папку `comp_name` в архів
                for foldername, subfolders, filenames in os.walk(comp_name_folder):
                    for filename in filenames:
                        filepath = os.path.join(foldername, filename)
                        arcname = os.path.relpath(filepath, comp_name_folder)
                        zipf.write(filepath, arcname)

            # Після створення архіву, видаляємо папку `comp_name`
            shutil.rmtree(comp_name_folder)

    except Exception as ex:
        print(f'Помилка створення архіву {ex}')
        continue

    # Передайте JWT токен у заголовку Authorization
    headers = {'Authorization': f'Bearer {jwt_token}'}

    # Встановіть ім'я, під яким ви хочете зберегти файл на СЕРВ1
    file_name_on_server = os.path.basename(comp) + '.zip'

    # Створіть об'єкт "файл" для відправки
    files = {'file': (file_name_on_server, open(zip_filename, 'rb'))}
    response = requests.post(server_url, files=files, headers=headers)


    if response.status_code == 200:
        try:
            data = response.json()
            #print(data)
        except json.decoder.JSONDecodeError:
            print("Помилка: Відповідь сервера не є дійсним JSON.")
    else:
        print(f"Помилка відповіді сервера: {response.status_code}.")
    print(response.json())